import React, { useState, useEffect } from 'react';
import axios from 'axios';
import { toast } from 'react-toastify';
import { BsX } from 'react-icons/bs';
import Querystring from 'querystring';
import jwtDecode from 'jwt-decode';

const useLocalStorage = (key, initialValue) => {
  const [storedValue, setStoredValue] = useState(() => {
    try {
      const item = window.localStorage.getItem(key);
      return item ? JSON.parse(item) : initialValue;
    } catch (error) {
      console.log(error);
      return initialValue;
    }
  });
  const setValue = value => {
    try {
      const valueToStore = value instanceof Function ? value(storedValue) : value;
      setStoredValue(valueToStore);
      window.localStorage.setItem(key, JSON.stringify(valueToStore));
    } catch (error) {
      console.log(error);
    }
  };
  return [storedValue, setValue];
};

const ruleRunner = (field, label, ...validations) => {
  return state => {
    for (let v of validations) {
      let errorMessageFunc = v(state[field], state);
      if (errorMessageFunc) {
        return {
          [field]: errorMessageFunc(label)
        };
      }
    }
    return null;
  };
};
const runValidation = (model, runners) => {
  return runners.reduce((memo, runner) => {
    return Object.assign(memo, runner(model));
  }, {});
};
const hasErrors = validationErrors => {
  for (var key in validationErrors) {
    if (validationErrors.hasOwnProperty(key)) {
      return true;
    }
  }
  return false;
};

var validationHook = ((initialModel, validationRules, submitHandler) => {
  const [model, setModel] = useState(initialModel);
  const [submitted, setSubmitted] = useState(false);
  const [errors, setErrors] = useState(false);
  const modelOnChange = (field, value) => {
    let newModel = {
      ...model
    };
    newModel[field] = value;
    setModel(newModel);
    setErrors(runValidation(newModel, validationRules));
  };
  function onSubmit(e) {
    e.preventDefault();
    setSubmitted(true);
    let errors = runValidation(model, validationRules);
    setErrors(errors);
    if (!hasErrors(errors)) {
      submitHandler(model);
    }
  }
  return [model, errors, submitted, modelOnChange, onSubmit];
});

const isRequiredMsg = fieldName => `${fieldName} is required`;
const mustMatchMsg = otherFieldName => {
  return fieldName => `${fieldName} and ${fieldName} Must match`;
};
const mustDifferMsg = otherFieldName => {
  return fieldName => `${fieldName} and ${otherFieldName} must differ`;
};
const minLengthMsg = length => {
  return fieldName => `${fieldName} must be ${length} lenght`;
};
const maxLengthMsg = length => {
  return fieldName => `${fieldName} must not have more than ${length} characters`;
};
const regexMsg = fieldName => {
  return `${fieldName} must match the appropriate pattern`;
};
const emailMsg = fieldName => `${fieldName} must be an email`;
const minValueMsg = fieldName => {
  return minValue => `${fieldName} must be greater than ${minValue}`;
};

const required = text => {
  if (text) {
    return null;
  } else {
    return isRequiredMsg;
  }
};
const mustMatch = (field, fieldName) => {
  return (text, state) => {
    return state[field] == text ? null : mustMatchMsg();
  };
};
const mustDiffer = (field, fieldName) => {
  return (text, state) => {
    return state[field] != text ? null : mustDifferMsg(fieldName);
  };
};
const minLength = length => {
  return text => {
    return (!text ? 0 : text.length) >= length ? null : minLengthMsg(length);
  };
};
const maxLength = length => {
  return text => {
    return (!text ? 0 : text.length) <= length ? null : maxLengthMsg(length);
  };
};
const regex = regex => {
  return text => {
    return regex.test(text) ? null : regexMsg;
  };
};
const email = text => {
  var re = /^$|^([\w\.\-]+)@([\w\-]+)((\.(\w){2,3})+)$/;
  return re.test(text) ? null : emailMsg;
};
const minValue = value => {
  return fieldValue => {
    return fieldValue && fieldValue >= value ? null : minValueMsg(value);
  };
};

var rules = {
    __proto__: null,
    required: required,
    mustMatch: mustMatch,
    mustDiffer: mustDiffer,
    minLength: minLength,
    maxLength: maxLength,
    regex: regex,
    email: email,
    minValue: minValue
};

var styles = {"loader":"_styles-module__loader__3qqrl","spin":"_styles-module__spin__1sp4O","loaderContainer":"_styles-module__loaderContainer__zQVVC","error":"_styles-module__error__3fyPz","modal":"_styles-module__modal__Xt-75","modalContent":"_styles-module__modalContent__3bARu","box":"_styles-module__box__37_FJ","boxHeader":"_styles-module__boxHeader__3m_dA","boxContent":"_styles-module__boxContent__1OV3o"};

var ErrorSpanComponent = (({
  field,
  errors
}) => {
  return !errors[field] || /*#__PURE__*/React.createElement("small", {
    className: styles.error
  }, errors[field]);
});

axios.interceptors.request.use(config => {
  return config;
}, error => {
  return Promise.reject(error);
});
axios.interceptors.response.use(response => {
  notify(response.headers);
  return response;
}, error => {
  let message = error.response && error.response.data.message ? error.response.data.message : error.message;
  toast.error(message);
  return Promise.reject(error);
});
const notify = headers => {
  if (headers['x-success-message']) {
    toast.success(headers['x-success-message']);
  }
  if (headers['x-info-message']) {
    toast.info(headers['x-info-message']);
  }
  if (headers['x-warning-message']) {
    toast.warning(headers['x-warning-message']);
  }
};
const updateAuthHeader = (authTokenName, token) => {
  let newToken = token || localStorage.getItem(authTokenName);
  if (newToken) {
    localStorage.setItem(authTokenName, newToken);
    axios.defaults.headers = {
      Authorization: `Bearer ${localStorage.getItem(authTokenName)}`
    };
  }
};
const clearAuthHeader = authTokenName => {
  localStorage.setItem(authTokenName, "");
};
const request = axios;

function Loader({
  threshold,
  isLoading
}) {
  let busy = false;
  const [isVisible, setVisible] = useState(false);
  useEffect(() => {
    if (isLoading) {
      setTimeout(() => {
        if (busy) {
          setVisible(true);
        }
      }, threshold);
    } else {
      setVisible(false);
      busy = false;
    }
  }, []);
  return isVisible ? /*#__PURE__*/React.createElement("div", {
    id: "loader",
    className: styles.loaderContainer
  }, /*#__PURE__*/React.createElement("div", {
    className: styles.loader
  })) : null;
}

var ModalComponent = (({
  children,
  visible,
  title,
  onClose
}) => {
  return visible && /*#__PURE__*/React.createElement("div", {
    className: styles.modal
  }, /*#__PURE__*/React.createElement("div", {
    className: styles.modalContent
  }, /*#__PURE__*/React.createElement("div", {
    className: styles.box
  }, /*#__PURE__*/React.createElement("div", {
    className: styles.boxHeader
  }, /*#__PURE__*/React.createElement("strong", null, title), /*#__PURE__*/React.createElement(BsX, {
    style: {
      cursor: 'pointer',
      float: 'right',
      paddingBottom: 4
    },
    size: 26,
    onClick: onClose
  }, "x")), /*#__PURE__*/React.createElement("div", {
    className: styles.boxContent
  }, children))));
});

class Auth2Service {
  constructor(authEndpoint, authTokenName) {
    this.authEndpoint = authEndpoint;
    this.authTokenName = authTokenName;
  }
  authenticate(clientId, clientSecret, email, password, callback, errorCallback) {
    let cliAuthData = `${clientId}:${clientSecret}`;
    let authData = Querystring.stringify({
      grant_type: 'password',
      username: email,
      password: password,
      scope: ['write', 'read']
    });
    let reqConfig = {
      headers: {
        'content-type': 'application/x-www-form-urlencoded',
        'X-Requested-With': 'XMLHttpRequest',
        Authorization: `Basic ${window.btoa(cliAuthData)}`
      }
    };
    request.post(this.authEndpoint, authData, reqConfig).then(response => {
      localStorage.setItem(this.authTokenName, response.data.access_token);
      updateAuthHeader(this.authTokenName, response.data.access_token);
      callback(response);
    }, errorCallback);
  }
  validateSession() {
    if (!localStorage.getItem(this.authTokenName)) {
      return false;
    }
    let authData = this.getSessionData();
    const date = new Date(0);
    date.setUTCSeconds(authData.exp);
    if (date.valueOf() > new Date().valueOf()) {
      updateAuthHeader(this.authTokenName);
      return true;
    } else {
      return false;
    }
  }
  getSessionData() {
    let sessionData = jwtDecode(localStorage.getItem(this.authTokenName));
    return sessionData;
  }
  signout(callback) {
    clearAuthHeader(this.authTokenName);
    if (callback) {
      callback();
    }
  }
}

const ErrorSpan = ErrorSpanComponent;
const useLocalStorage$1 = useLocalStorage;
const useValidation = validationHook;
const validationRules = rules;
const ruleRunner$1 = ruleRunner;
const hasErrors$1 = hasErrors;
const http = request;
const Loader$1 = Loader;
const updateAuthHeader$1 = updateAuthHeader;
const clearAuthHeader$1 = clearAuthHeader;
const Modal = ModalComponent;
const Auth2Service$1 = Auth2Service;

export { Auth2Service$1 as Auth2Service, ErrorSpan, Loader$1 as Loader, Modal, clearAuthHeader$1 as clearAuthHeader, hasErrors$1 as hasErrors, http, ruleRunner$1 as ruleRunner, updateAuthHeader$1 as updateAuthHeader, useLocalStorage$1 as useLocalStorage, useValidation, validationRules };
//# sourceMappingURL=index.modern.js.map
